// File:      MeshEggs.h
// Author:    Philip Rideout
// Brief:     Control data for various subdivision shapes.  This file should be included only once.

struct MeshEgg
{
    unsigned int pointCount;
    float* points;

    // faces (four indices each)
    unsigned int faceCount;
    unsigned int* faces;

    // tagged edges (two indices each)
    // all boundary edges must be tagged
    // all edges that have a tagged vertex must be tagged
    // TODO can these be derived?
    unsigned int creaseEdgeCount;
    unsigned int* creaseEdges;

    // tagged vertices (one index each)
    // all vertices that have an incident tagged edge must be tagged in one of the three ways:

    // crease vertex: joins exactly two incident crease edges smoothly
    unsigned int creaseVertexCount;
    unsigned int* creaseVertices;

    // corner vertex: connects two or more creases in a corner (convex or concave)
    unsigned int cornerVertexCount;
    unsigned int* cornerVertices;

    // dart vertex: causes the crease to blend smoothly into the surface
    unsigned int dartVertexCount;
    unsigned int* dartVertices;
};

#define T (1.0f / 3.0f)
float gizmoPoints[] =
{
    -T,  1, -T,
     T,  1, -T,
     T,  1,  T,
    -T,  1,  T,

    -T,  T, -T,
     T,  T, -T,
     T,  T,  T,
    -T,  T,  T,

    -T, -T, -T,
     T, -T, -T,
     T, -T,  T,
    -T, -T,  T,

    -T, -1, -T,
     T, -1, -T,
     T, -1,  T,
    -T, -1,  T,

    -1,  T, -T,
    -1,  T,  T,
    -1, -T,  T,
    -1, -T, -T,

     1,  T, -T,
     1,  T,  T,
     1, -T,  T,
     1, -T, -T,

    -T, -T, -1,
    -T,  T, -1,
     T,  T, -1,
     T, -T, -1,

    -T, -T,  1,
    -T,  T,  1,
     T,  T,  1,
     T, -T,  1
};
#undef T

unsigned int gizmoFaces[] =
{
    0, 1, 5, 4,
    1, 2, 6, 5,
    2, 3, 7, 6,
    3, 0, 4, 7,

    16, 4, 8, 19,
    17, 7, 4, 16,
    18, 11, 7, 17,
    19, 8, 11, 18,

    5, 20, 23, 9,
    6, 21, 20, 5,
    10, 22, 21, 6,
    9, 23, 22, 10,

    8, 4, 25, 24,
    4, 5, 26, 25,
    5, 9, 27, 26,
    9, 8, 24, 27,

    28, 29, 7, 11,
    29, 30, 6, 7,
    30, 31, 10, 6,
    31, 28, 11, 10,

    8, 9, 13, 12,
    9, 10, 14, 13,
    10, 11, 15, 14,
    11, 8, 12, 15
};

unsigned int gizmoCreaseEdges[] =
{
    0, 1,
    1, 2,
    2, 3,
    3, 0,

    12, 13,
    13, 14,
    14, 15,
    15, 12,

    16, 17,
    17, 18,
    18, 19,
    19, 16,

    20, 21,
    21, 22,
    22, 23,
    23, 20,

    24, 25,
    25, 26,
    26, 27,
    27, 24,

    28, 29,
    29, 30,
    30, 31,
    31, 28,
};

unsigned int gizmoCreaseVertices[] =
{
    0, 1, 2, 3,
    12, 13, 14, 15,
    16, 17, 18, 19,
    20, 21, 22, 23,
    24, 25, 26, 27,
    28, 29, 30, 31
};

MeshEgg gizmo =
{
    sizeof(gizmoPoints) / (sizeof(float) * 3),
    gizmoPoints,
    sizeof(gizmoFaces) / (sizeof(unsigned int) * 4),
    gizmoFaces,
    sizeof(gizmoCreaseEdges) / (sizeof(unsigned int) * 2),
    gizmoCreaseEdges,
    sizeof(gizmoCreaseVertices) / (sizeof(unsigned int)),
    gizmoCreaseVertices,
    0, 0, // no corners
    0, 0  // no darts
};
