// File:      Platform.h
// Author:    Philip Rideout
// Brief:     Includes GLEW and declares all functions that have platform-specific implementations.

#pragma once

#include <GL/glew.h>

// GLEW 1.3.2 defines the wrong constant
#undef GL_SHADER_TYPE
#define GL_SHADER_TYPE 0x8B4F

// miscelleneous platform-specific macros
#define MAX_ERROR_LENGTH 256
#define MAX_PATH_LENGTH 256
#define INVALID_TLS_INDEX ((tls_index) 0xffffffff)

#define OSTHREADRETTYPE unsigned long
#define OSTHREADCALLCONV __stdcall
#define OSTHREADFUNC OSTHREADRETTYPE OSTHREADCALLCONV

// miscelleneous platform-specific functions
int osGetScreenHeight(); // in pixels
int osGetScreenWidth(); // in pixels
int osGetHorizRes(); // in dpi
int osGetVertRes(); // in dpi
double osGetSeconds();
void osError(const char* message);
void osBeep();
char* osGetFontPath(const char* base);

typedef void* OsThreadHandle;
typedef OSTHREADRETTYPE (OSTHREADCALLCONV *OsThreadFunc)(void*);
OsThreadHandle osFork(OsThreadFunc, void*);
void osCloseThread(OsThreadHandle);
void osYield();
void osProcessMessages();
